package com.twelvemonkeys.imageio.plugins.pict;

import com.twelvemonkeys.image.BufferedImageIcon;
import com.twelvemonkeys.image.ImageUtil;
import com.twelvemonkeys.net.MIMEUtil;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.stream.ImageInputStream;
import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;

/**
 * TestPICTClippingApp
 *
 * @author <a href="mailto:harald.kuhr@gmail.com">Harald Kuhr</a>
 * @author last modified by $Author: haraldk$
 * @version $Id: TestPICTClippingApp.java,v 1.0 Feb 16, 2009 3:05:16 PM haraldk Exp$
 */
public class TestPICTClippingApp {
    public static void main(final String[] pArgs) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
                    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                }
                catch (Exception e) {
                    throw new RuntimeException(e);
                }

                JFrame frame = new JFrame("PICTClipping test");
                frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                JLabel dropZone = new JLabel("Drop images here", JLabel.CENTER) {
                    @Override
                    public Dimension getPreferredSize() {
                        return new Dimension(320, 200);
                    }
                };

                dropZone.setTransferHandler(new ImageDropHandler(dropZone));
                frame.add(dropZone);
                frame.pack();
                frame.setLocationRelativeTo(null);
                frame.setVisible(true);
            }
        });
    }

    private static class ImageDropHandler extends TransferHandler {
        private final JLabel mLabel;

        public ImageDropHandler(JLabel pLabel) {
            super(null);
            mLabel = pLabel;
        }

        private DataFlavor getSupportedFlavor(DataFlavor[] transferFlavors) {
            for (DataFlavor flavor : transferFlavors) {
                String type = MIMEUtil.bareMIME(flavor.getMimeType());
                if (InputStream.class.isAssignableFrom(flavor.getDefaultRepresentationClass()) && ImageIO.getImageReadersByMIMEType(type).hasNext()) {
                    return flavor;
                }
                else if (flavor.equals(DataFlavor.javaFileListFlavor)) {
                    return flavor;
                }
            }

            for (DataFlavor flavor : transferFlavors) {
                System.err.println("flavor: " + flavor);
            }

            return null;
        }

        @Override
        public boolean canImport(JComponent comp, DataFlavor[] transferFlavors) {
            return getSupportedFlavor(transferFlavors) != null;
        }

        @Override
        public boolean importData(JComponent comp, Transferable t) {
            DataFlavor[] flavors = t.getTransferDataFlavors();
            DataFlavor flavor = getSupportedFlavor(flavors);
            if (flavor != null) {
                try {
                    InputStream input;
                    if (DataFlavor.javaFileListFlavor.equals(flavor)) {
                        List files = (List) t.getTransferData(flavor);
                        if (files.isEmpty()) {
                            return false;
                        }
                        else {
                            input = new FileInputStream((File) files.get(0));
                        }
                    }
                    else {
                        Object data = t.getTransferData(flavor);
                        input = (InputStream) data;
                    }

                    ImageInputStream stream = ImageIO.createImageInputStream(input);
                    Iterator<ImageReader> readers = ImageIO.getImageReaders(stream);

                    if (!readers.hasNext()) {
                        readers = ImageIO.getImageReadersByMIMEType(MIMEUtil.bareMIME(flavor.getMimeType()));
                    }

                    if (readers.hasNext()) {
                        ImageReader reader = readers.next();
                        reader.setInput(stream);
                        BufferedImage image = reader.read(0);

                        mLabel.setIcon(new BufferedImageIcon(box(image, 200)));

                        return true;
                    }
                    else {
                        System.err.println("No reader found!");
                    }

                }
                catch (UnsupportedFlavorException ignore) {
                    ignore.printStackTrace();
                }
                catch (IOException ignore) {
                    ignore.printStackTrace();
                }
                catch (RuntimeException e) {
                    e.printStackTrace();
                    throw e;
                }
            }

            return false;
        }

        private BufferedImage box(final BufferedImage pImage, final int pMaxDimension) {
            if (pImage.getWidth() > 200 || pImage.getHeight() > 200) {
                int w, h;

                if (pImage.getWidth() > pImage.getHeight()) {
                    w = pMaxDimension;
                    h = (int) Math.round(w / (pImage.getWidth() / (double) pImage.getHeight()));
                }
                else {
                    h = pMaxDimension;
                    w = (int) Math.round(h * (pImage.getWidth() / (double) pImage.getHeight()));
                }

                return ImageUtil.createResampled(pImage, w, h, Image.SCALE_DEFAULT);
            }
            return pImage;
        }
    }
}